<?php

    /**
     * This class handles customer and salers order sync
     * This class uses order action "woocommerce_checkout_order_processed" which is 
     * triggered right before payment method triggers.This class overwrite the 
     * woocommerce payment calls and calls payment method.If payment call is 
     * successful then it transfer sales order to Netsuite.
     */
    class Admin_Manual_Sync {

        public $is_purchase_order = FALSE;

        public function __construct() {

            $co_options = In8_SalesOrder::getOptions(2);

            if (isset($co_options['enable_so_sync']) && $co_options['enable_so_sync'] == 'on') {
                //add admin meta box
                add_action('add_meta_boxes', array($this, 'add_meta_box'));
	            add_action('admin_enqueue_scripts', function(){
		            wp_enqueue_script('admin-script-so', In8_SalesOrder_URL . "inc/assets/js/admin-script.js");
		            wp_enqueue_style('admin-style-so', In8_SalesOrder_URL . "inc/assets/css/admin-style.css");
	            });
                
                //ajax for pushing order to netsuite
                add_action('wp_ajax_push_order_manually', array($this, 'pushOrderManually'));
                add_action('wp_ajax_nopriv_push_order_manually', array($this, 'pushOrderManually'));
            }
        }

        public function add_meta_box() {
            add_meta_box('woocommerce-netsuite-order-sync', __('In8Sync NetSuite Synchronization', 'woocommerce-netsuite-order-sync'), array($this, 'meta_box'), 'shop_order', 'side', 'high');
        }

        /**
         * Show the meta box for shipment info on the order page
         *
         * @access public
         */
        public function meta_box() {
            global $post;

            $sales_order_internalID = get_post_meta($post->ID, 'internalid', TRUE);
            $sales_order_number = get_post_meta($post->ID, 'salesordernumber', TRUE);

            
			echo "<p><a class='thickbox' name='NetSuite Order Request' href='?in8_get_order_json_request=1&order=". $post->ID ."'>Show Request Data</a> | <a class='thickbox' href='?view_in8_log_type=log&view_in8_log_object=product&view_in8_log_object_id=". $post->ID ."'>Load logs</a></p>";
	        do_action("in8_so_order_meta_box_after_logs", "", $post->ID);
			if (empty($sales_order_internalID)) {
                echo "<p><span class='replace-content'>This order has not been synced with NetSuite.</span><br/><br/> <a data-order_id='" . $post->ID . "' data-admin_ajax='" . admin_url('admin-ajax.php') . "' href='javascript:void(0)' id='manualOrderPush'>Click here</a> to sync it.</p>";
            } else {
                //echo "<p><span class='replace-content'>This order has been successfully transferred to NetSuite.</span><br/><br/>NetSuite Sales Order Internal Id: <b>" . $sales_order_internalID . "</b><br/><br/> <a data-order_id='" . $post->ID . "' data-admin_ajax='" . admin_url('admin-ajax.php') . "' href='javascript:void(0)' id='manualOrderPush'> Click Here</a> to update Order on NetSuite<p>";

				$id = get_post_meta($post->ID, 'internalid', TRUE);
				$settings = IN8_Settings::getTabSettings('in8_tab_general', 'ns_api_creds');
				$url = $settings['restlet_url'];
				if( !empty($url) && !empty($id) ){
					$parse = parse_url($url);
					$order_link = 'https://'.$parse['host'].'/app/accounting/transactions/salesord.nl?id='.$id.'&whence=';
					$order_link = str_replace('restlets.api', 'app', $order_link);
					$order_link = apply_filters('in8sync_so_ns_url', $order_link, $id, $post->ID);
					$sales_order_number = "<a target='_blank' href='{$order_link}'>{$sales_order_number}</a>";
				}

                echo "<table><tr><td style='vertical-align:top'><div class='greenCircle'></div></td>
                <td>&nbsp;&nbsp;</td><td><span class='replace-content'>
                This order has been successfully transferred to NetSuite.</span><br/><br/></td></tr></table>NetSuite Sales Order: <b>" . $sales_order_number . 
                "</b><br/><br/> <a data-order_id='" . $post->ID . "' data-admin_ajax='" . admin_url('admin-ajax.php') . "' href='javascript:void(0)' id='manualOrderPush'> Click Here</a> to update Order on NetSuite.";
            }

            echo " <div id='bowlG' style='display:none;' class='loader'></div>";
            echo " <div id='divProcessing' style='display:none;'>Syncing order to NetSuite. Please wait.</div>";

            //echo " <div id='bowlG' style='display:none;'>
	    //<div id='bowl_ringG'>
		//<div class='ball_holderG'>
		//	<div class='ballG'>
		//	</div>
		//</div></div></div>";
        }

        private function isPurchaseOrder($order_id) {

            $active_plugins = apply_filters('active_plugins', get_option('active_plugins'));

            if (in_array('in8sync-purchaseorder/in8sync-purchaseorder.php', $active_plugins)) {
                $plugin_conf_options = In8_PurchaseOrder::getOptions(2);

                if (isset($plugin_conf_options['all_purchase_orders'])) {
                    $this->is_purchase_order = TRUE;
                }

                if (!$this->is_purchase_order && isset($plugin_conf_options['identify_order_type']) && $plugin_conf_options['identify_order_type'] == 'on') {
                    $order_type = get_post_meta($order_id, $plugin_conf_options['identify_order_type_custom_field'], TRUE);

                    if ($order_type == $plugin_conf_options['identify_purchase_order_custom_field_value']) {
                        $this->is_purchase_order = TRUE;
                    }
                }
            }

            if ($this->is_purchase_order) {
                remove_action('woocommerce_order_status_processing', [$this, 'sendOrderToNetsuite'], 1, 1);
                remove_action('woocommerce_order_status_on-hold', [$this, 'sendOrderToNetsuite'], 1, 1);
            }

            return $this->is_purchase_order;
        }

        public function pushOrderManually() {
            


            $return['status'] = 0;
            $order_id = $_GET['order_id'];

            if (!empty($order_id)) {
                $orderInternalId = get_post_meta($order_id, 'internalid', TRUE);

                $order_type = 'sales';
                $order_message = '';
                if (empty($orderInternalId)) { // a new order create request
                    if ($this->isPurchaseOrder($order_id)) {
                        $In8_Vendors_Purchase_Sync = new In8_Vendors_Purchase_Sync();
                        $In8_Vendors_Purchase_Sync->newOrderCreated($order_id);
                        $order_co_options = In8_PurchaseOrder::getOptions(2);
                        if (!isset($order_co_options['sync_on_creation'])) {
                            $In8_Vendors_Purchase_Sync->sendOrderToNetsuite($order_id, 0, TRUE);
                        }
                        $order_message = "New purchase order create";
                        $order_type = 'purchase';
                    } else {
                        $In8_Customers_Sales_Sync = In8_Customers_Sales_Sync::getInstance();
                        $In8_Customers_Sales_Sync->newOrderCreated($order_id);
                        $order_co_options = In8_SalesOrder::getOptions(2);
                        if (!isset($order_co_options['sync_on_creation'])) {
                            $In8_Customers_Sales_Sync->sendOrderToNetsuite($order_id, 0, TRUE);
                        }

                        $order_message = "New sales order create";
                    }
                    $orderInternalId = get_post_meta($order_id, 'internalid', TRUE);
                } else { //order update request
                    $check_purchase = get_post_meta($order_id, 'purchaseordernumber', TRUE);
                    if (!empty($check_purchase)) { //a purchase order
                        $In8_Vendors_Purchase_Sync = new In8_Vendors_Purchase_Sync();
                        $In8_Vendors_Purchase_Sync->sendOrderToNetsuite($order_id, $orderInternalId, TRUE);
                        $order_message = "Existing purchase order update";
                        $order_type = 'purchase';
                    } else {
                        $In8_Customers_Sales_Sync = In8_Customers_Sales_Sync::getInstance();
                        $In8_Customers_Sales_Sync->sendOrderToNetsuite($order_id, $orderInternalId);
                        $order_message = "Existing sales order update";
                    }
                }

                if (empty($orderInternalId)) { //order doesn't exists on Netsuite 
                    $return['message'] = "There is an error syncing this order.";
                    if ($order_type == 'sales') {
                        $object = "salesorder";
                        $log_file = IN8_Settings::getLogFileURL('log', 'salesorder', $order_id);
                        if ($log_file === FALSE) {
                            $log_file = IN8_Settings::getLogFileURL('log', 'salesorder-customer', $order_id);
                            $object = "salesorder-customer";
                            if ($log_file === FALSE) {
                                $log_file = IN8_Settings::getLogFileURL('log', 'salesorder-customer-search', $order_id);
                                $object = "salesorder-customer-search";
                            }
                        }
                    } elseif ($order_type == 'purchase') {
                        $log_file = IN8_Settings::getLogFileURL('log', 'purchaseorder', $order_id);
                        $object = "purchaseorder";
                        if ($log_file === FALSE) {
                            $log_file = IN8_Settings::getLogFileURL('log', 'purchaseorder-vendor', $order_id);
                            $object = "purchaseorder-vendor";
                            if ($log_file === FALSE) {
                                $log_file = IN8_Settings::getLogFileURL('log', 'purchaseorder-vendor-search', $order_id);
                                $object = "purchaseorder-vendor-search";
                            }
                        }
                    }

                    if (isset($log_file)) {
                        $return['message'].= "<a href='?download_in8_log_type=log&download_in8_log_object=" . $object . "&download_in8_log_object_id=" . $order_id . "'>Click Here</a> to download the error log file.";
                    }
                } else {
                    $return['status'] = 1;
                    $return['message'] = $order_message . " request was successful";
                }

                echo json_encode($return);
                die;
            }
        }

    }

    new Admin_Manual_Sync();
    