<?php
	
	// If the transaction id is missing set the order to sync async
	class In8_Customers_Sales_Options {
		public function __construct() {
			
			//add_filter('in8_do_not_sync', [$this, 'fix_missing_transaction_id'], 10, 3);
			
			$this->load_save_internalid_on_main_order_endpoint();
			
			$this->show_internal_id_filter();
			
			// this will probably be needed to save the address id
			$this->add_name_to_multiple_addresses_field();
			
			// this will save the generic items
			$this->save_generic_item();
			
			// this will map the external id on the salesorder
			$this->mapExternalIdOnSalesOrder();
			
			$this->save_sub_instance_id();
			
			$this->stop_fields_from_sending_to_ns();
			
			// this will return the json request that will be sent to ns for a specific order
			$this->get_order_json_request_to_ns();
			
			// this will help with shipping methods mapping for ups and fedex
			$this->get_ups_fedex_shipping_methods_mappings();
			
			
			// option to stop order from re-syncing to NS
			$this->stop_order_from_re_syncing();
			
			
			// map the gravity form pdf
			$this->map_gravity_form_pdf();
			
			
			// map the has_role field
			$this->map_has_role();
			
			// show internalid warning on SO page
			$this->show_internal_id_warning_on_so_page();
			
			
			// Customization map payeezy credit card payment method
			$this->map_payeezy_credit_card();
			
			// default message for do not sync
			add_filter('in8_do_not_sync_error', function(){return 'Stopped by doNotSync.';});
			add_filter('in8_send_error_notification', function($default, $code){
				return $code == '0000' ? false : $default;
			}, 10, 2);
			
			// Customization map cybersource credit card payment method
			$this->map_cybersource_credit_card();
			
			// Customization map new cybersource tokens
			$this->map_new_cybersource_tokens();
			
			// deal with customer syncing tokens
			$this->sync_customer_with_tokens();
			
			
			// support for WooThumbs for WooCommerce by Iconic
			add_filter('in8_product_additional_variation_images', function($default){
				if( in_array('woothumbs-premium/iconic-woothumbs.php', get_option('active_plugins')) ) {
					return 'variation_image_gallery';
				}
				return $default;
			});
			
		}
		
		/**
		 * If using payment methods that use transaction id
		 * Use this function to move the sync to async if the transaction id is missing
		 *
		 * @param $default
		 * @param $order_id
		 * @param $type
		 *
		 * @return bool
		 */
		function fix_missing_transaction_id( $default, $order_id, $type ) {
			if ( $type != 'order' ) {
				return $default;
			}
			
			$paymentMethodsToCheck = apply_filters( 'in8_fix_missing_transaction_id', [ 'cybersource' ], $order_id );
			
			$order = wc_get_order( $order_id );
			if ( in_array( $order->get_payment_method(), $paymentMethodsToCheck ) ) {
				$transaction_id = $order->get_meta( '_transaction_id', TRUE );
				if ( empty( $transaction_id ) ) {
					wp_schedule_single_event( time() + 30, 'in8sync_async_sendOrderToNetsuite', [ $order_id ] );
					
					return TRUE;
				}
			}
			
			return $default;
		}
		
		
		function load_save_internalid_on_main_order_endpoint() {
			add_action( 'woocommerce_api_create_order', [ $this, 'save_internalid_on_main_order_endpoint' ], 10, 2 );
			add_action( 'woocommerce_api_edit_order', [ $this, 'save_internalid_on_main_order_endpoint' ], 10, 2 );
		}
		
		/**
		 * Feature to save the internalid directly from the order endpoint
		 *
		 * @param $order_id
		 * @param $data
		 */
		function save_internalid_on_main_order_endpoint( $order_id, $data ) {
			if ( isset( $data['internalid'] ) ) {
				update_post_meta( $order_id, 'internalid', $data['internalid'] );
			}
		}
		
		
		public static $order_statuses = [
//			'wc-pending',
			'wc-processing',
			'wc-completed',
		];
		
		function show_internal_id_filter() {
			
			add_filter( 'views_edit-shop_order', function ( $views ) {
				
				$count = "(" . In8_Customers_Sales_Options::getUnSyncedCount() . ")";
				$field = '<a href="' . admin_url( 'edit.php?post_type=shop_order&un-synced=1' ) . '">Unsynced Orders ' . $count . '</a>';
				if ( isset( $_REQUEST['un-synced'] ) ) {
					$field = 'Unsynced Orders ' . $count;
				}
				
				$views['un-synced'] = $field;
				
				return $views;
				
			} );
			add_filter( 'pre_get_posts', function ( $query ) {
				global $pagenow;
				
				if ( 'edit.php' != $pagenow || ! $query->is_admin ) {
					return $query;
				}
				
				if ( isset( $_REQUEST['un-synced'] ) && isset( $_REQUEST['post_type'] ) && $_REQUEST['post_type'] == 'shop_order' ) {
					/* @var WP_Query */
					$meta_query = [
						[
							'key'     => 'internalid',
							'compare' => 'NOT EXISTS',
						],
					];
					$statuses   = apply_filters( 'in8_so_order_statuses', In8_Customers_Sales_Options::$order_statuses );
					
					$query->set( 'post_status', $statuses );
					$query->set( 'meta_query', $meta_query );
					
					// how far should we look back
					$postDate = self::getUnSyncedDateInterval();
					if( !empty($postDate) ) {
						$query->set( 'date_query', [
							[
								'after'     => $postDate,
								'inclusive' => TRUE,
							]
						] );
					}
				}
				
				return $query;
			} );
		}
		
		static function getUnSyncedDateInterval() {
			$co_options = In8_SalesOrder::getOptions(2);
			$unsynced_orders_track    = isset($co_options['unsynced_orders_track']) ? $co_options['unsynced_orders_track'] : '';
			if( !empty( $unsynced_orders_track ) ){
				if( $unsynced_orders_track == '1yr' ){
					$postDateTime = strtotime("-1 year");
				}elseif( $unsynced_orders_track == '6mo' ) {
					$postDateTime = strtotime("-6 months");
				}elseif( $unsynced_orders_track == '1mo' ) {
					$postDateTime = strtotime("-1 month");
				}elseif( $unsynced_orders_track == '1wk' ) {
					$postDateTime = strtotime("-1 week");
				}
				
				return date('Y-m-d h:i:s', $postDateTime);
			}
			
			return false;
		}
		
		static function getUnSyncedCount( $return_count = TRUE ) {
			
			$post_statuses = apply_filters( 'in8_so_order_statuses', In8_Customers_Sales_Options::$order_statuses );
			$post_statuses = implode( "','", $post_statuses );
			
			// how far should we look back
			$postDate = self::getUnSyncedDateInterval();
			$unsynced_orders_interval_sql = !empty($postDate) ?  "AND posts.post_date > '{$postDate}'" : '';
			
			global $wpdb;
			if ( $return_count ) {
				$result = [];
				if ( apply_filters( 'in8_get_unsynced_orders_transient', FALSE ) ) {
					$result = get_transient( "in8_get_unsynced_orders_list" );
				}
				if ( empty( $result ) ) {
					$result = $wpdb->get_var( "
						SELECT COUNT(*) as counter FROM {$wpdb->posts} as posts
						WHERE   posts.post_type     = 'shop_order' AND posts.post_status IN ('{$post_statuses}')
						AND NOT EXISTS (
						  SELECT * FROM `{$wpdb->postmeta}` as postmeta
						   WHERE `postmeta`.`meta_key` = 'internalid'
							AND `postmeta`.`post_id`=posts.ID
						)
						{$unsynced_orders_interval_sql}
					" );
					
					if ( apply_filters( 'in8_get_unsynced_orders_transient', FALSE ) ) {
						set_transient( "in8_get_unsynced_orders_list", $result, 60 * 15 );
					}
				}
			} else {
				$items  = $wpdb->get_results( "
					SELECT ID FROM {$wpdb->posts} as posts
					WHERE   posts.post_type     = 'shop_order' AND posts.post_status IN ('{$post_statuses}')
					AND NOT EXISTS (
					  SELECT * FROM `{$wpdb->postmeta}` as postmeta
					   WHERE `postmeta`.`meta_key` = 'internalid'
						AND `postmeta`.`post_id`=posts.ID
					)
					{$unsynced_orders_interval_sql}
				" );
				$result = [];
				foreach ( $items as $item ) {
					$result[] = $item->ID;
				}
			}
			
			return $result;
		}
		
		
		public function add_name_to_multiple_addresses_field() {
			add_action( 'woocommerce_checkout_create_order', function ( $order ) {
				if ( isset( $_REQUEST['wcmca_address_select_menu_billing'] ) || isset( $_REQUEST['wcmca_address_select_menu_shipping'] ) ) {
					$order->update_meta_data( 'billing_address_id', $_REQUEST['wcmca_address_select_menu_billing'] );
					$order->update_meta_data( 'shipping_address_id', $_REQUEST['wcmca_address_select_menu_shipping'] );
				}
			} );
		}
		
		
		/**
		 * This feature allows you to use a generic item for items that do not exist on the webStore
		 */
		public $genericItemId;
		public $in8_saved_line_items = [];
		
		public static function getGenericItem() {
			$order_co_options = In8_SalesOrder::getOptions( 2 );
			if ( empty( $order_co_options['generic_item'] ) ) {
				return 0;
			}
			
			return $order_co_options['generic_item'];
		}
		
		public function save_generic_item() {
			// generic item
			$order_co_options = In8_SalesOrder::getOptions( 2 );
			if ( empty( $order_co_options['generic_item'] ) ) {
				return;
			}
			
			$this->genericItemId = $order_co_options['generic_item'];
			add_filter( 'woocommerce_api_create_order_data', [ $this, 'api_collect_line_items' ] );
			add_filter( 'woocommerce_api_edit_order_data', [ $this, 'api_collect_line_items' ] );
			
			
			add_action( 'woocommerce_api_create_order', [ $this, 'api_saved_order' ], 10, 2 );
			add_action( 'woocommerce_api_edit_order', [ $this, 'api_saved_order' ], 10, 2 );
			
			// for order_items endpoing
			add_filter( 'in8_endpoint_order_items_lines', [ $this, 'endpoint_order_items_lines' ], 10, 2 );
		}
		
		public function api_collect_line_items( $data ) {
			$this->in8_saved_line_items = [];
			if ( isset( $data['line_items'] ) ) {
				$line_items = $data['line_items'];
				foreach ( $line_items as $key => $item ) {
					if ( $item['product_id'] == $this->genericItemId ) {
						$this->in8_saved_line_items[] = $item;
						unset( $data['line_items'][ $key ] );
					}
				}
			}
			
			return $data;
		}
		
		public function api_saved_order( $order_id ) {
			if ( ! empty( $this->in8_saved_line_items ) ) {
				$order = wc_get_order( $order_id );
				foreach ( $this->in8_saved_line_items as $itemData ) {
					/* @var WC_Order_item $item */
					$item = new WC_Order_Item_Product();
					if ( isset( $itemData['id'] ) ) {
						$item = $order->get_item( $itemData['id'] );
					}
					
					$item->set_props( $itemData );
					
					$genericItemTitle = ! empty( $itemData['description'] ) ? $itemData['description'] : $itemData['displayname'];
					if ( ! empty( $genericItemTitle ) ) {
						$item->set_name( $genericItemTitle );
					}
					$item->add_meta_data( 'sku', $itemData['_sku'] );
					$item->add_meta_data( 'internalid', $itemData['internalid'] );
					
					$item = apply_filters( 'in8_order_save_generic_item', $item, $itemData, $order_id );
					$item->save();
					
					if ( ! isset( $itemData['id'] ) ) {
						$order->add_item( $item );
					}
					
					$order->save();
				}
			}
		}
		
		public function endpoint_order_items_lines( $lines, $order ) {
			/* @var WC_Order $order */
			
			$this->in8_saved_line_items = [];
			if ( ! empty( $lines ) ) {
				foreach ( $lines as $key => $item ) {
					if ( isset( $item['line_type'] ) && $item['line_type'] != 'line_item' ) {
						continue;
					}
					
					if ( $item['product_id'] == $this->genericItemId ) {
						$this->in8_saved_line_items[] = $item;
						unset( $lines[ $key ] );
					}
				}
				
				if ( ! empty( $this->in8_saved_line_items ) ) {
					$this->api_saved_order( $order->get_id() );
				}
			}
			
			return $lines;
		}
		
		
		/**
		 *    This function will check if the use of external id is on
		 *    If it is on it will grab the prefix that was saved or default to wc_
		 *    It will map the externalid field to the prefix and order id, eg: wc_123
		 */
		public function mapExternalIdOnSalesOrder() {
			add_filter( 'in8sync_so_order_after_data_prepared', function ( $default, $order_id ) {
				// generic item
				$order_co_options = In8_SalesOrder::getOptions( 2 );
				if ( empty( $order_co_options['use_external_id'] ) ) {
					return $default;
				}
				
				$prefix                = empty( $order_co_options['prefix_externalid'] ) ? 'wc_' : $order_co_options['prefix_externalid'];
				$externalId            = $prefix . $order_id;
				$default['externalid'] = $externalId;
				
				return $default;
			}, 10, 2 );
		}
		
		public function save_sub_instance_id() {
			add_action( 'woocommerce_checkout_create_order_shipping_item', function ( $item, $package_key, $package ) {
				$chosen_shipping_methods = WC()->session->get( 'chosen_shipping_methods' );
				/* @var WC_Shipping_Rate $selected_shipping_method */
				/* @var WC_Order $order */
				
				if ( empty( $chosen_shipping_methods ) ) {
					return;
				}
				if ( ! isset( $chosen_shipping_methods[ $package_key ] ) ) {
					return;
				}
				if ( empty( $package ) || empty( $package['rates'] ) || empty( $package['rates'][ $chosen_shipping_methods[ $package_key ] ] ) ) {
					return;
				}
				
				$selected_shipping_method = $package['rates'][ $chosen_shipping_methods[ $package_key ] ];
				$id                       = $selected_shipping_method->get_id();
				$id_parts                 = explode( ':', $id );
				if ( count( $id_parts ) == 3 ) {// if it has a sub id
					$item->add_meta_data( 'instance_sub_id', $id_parts[2], TRUE );
				} else if (
					count( $id_parts ) == 2 &&
					(
						! is_numeric( $id_parts[1] ) // if the instance id is text save it as sub instance id
						|| ( ! empty( $id_parts[1] ) && $id_parts[1][0] == 0 ) // if the instance id begins with 0, eg: 01, 09...
						|| $id_parts[0] == 'ups'
					)
				) {
					$item->add_meta_data( 'instance_sub_id', $id_parts[1], TRUE );
				}else if( $selected_shipping_method->get_method_id() == 'webship' ){
					$item->add_meta_data( 'instance_sub_id', $id, TRUE );
				}
			}, 10, 3 );
			add_filter( 'woocommerce_hidden_order_itemmeta', function ( $default ) {
				$default[] = 'instance_sub_id';
				
				return $default;
			} );
		}
		
		public function stop_fields_from_sending_to_ns() {
			add_filter( 'in8sync_so_order_after_data_prepared', function ( $data ) {
				$order_co_options = In8_SalesOrder::getOptions( 2 );
				if ( ! empty( $order_co_options['stop_fields_from_syncing'] ) ) {
					foreach ( $order_co_options['stop_fields_from_syncing'] as $field ) {
						if ( isset( $data[ $field ] ) ) {
							unset( $data[ $field ] );
						}
					}
				}
				
				return $data;
			} );
		}
		
		public function get_order_json_request_to_ns() {
			
			add_action( 'init', function () {
				if ( ! isset( $_REQUEST['in8_get_order_json_request'] ) || ! isset( $_REQUEST['order'] ) ) {
					return;
				}
				
				$order = wc_get_order( $_REQUEST['order'] );
				if ( ! $order ) {
					die( 'Something went wrong' );
				}
				
				
				//plugin options
				$plugin_options = In8_SalesOrder::getOptions();
				
				$SO = In8_Customers_Sales_Sync::getInstance();
				$SO->set_order_id( $order->get_id() );
				$SO->set_user_id( $order->get_customer_id() );
				$SO->set_customer_internal_id( get_post_meta( $order->get_id(), 'customerinternalid', TRUE ) );
				$order_internal_id = $order->get_meta( 'internalid', TRUE );
				$customer_object            = $SO->_prepareCustomerData( $order, $plugin_options );
				$object            = $SO->_getOrderData( $order, $plugin_options, $order_internal_id );
				?>
				<h3>Customer Request</h3>
				<pre><?php print_r( json_encode( $customer_object, JSON_PRETTY_PRINT ) ); ?></pre>
				<h3>Order Request</h3>
				<pre><?php print_r( json_encode( $object, JSON_PRETTY_PRINT ) ); ?></pre>
				<?php
				exit;
			} );
		}
		
		
		public function get_ups_fedex_shipping_methods_mappings() {
			add_filter( 'in8_so_shipping_methods_default', function ( $default ) {
				
				$wc_shipping_method_classes = [
					'ups'   => 'WC_Shipping_UPS',
					'fedex' => 'WC_Shipping_Fedex',
				];
				foreach ( $wc_shipping_method_classes as $method_id => $class_name ) {
					if ( ! class_exists( $class_name ) ) {
						continue;
					}
					
					$reader = function & ( $object, $property, $args = [] ) {
						$value = &Closure::bind( function & () use ( $property, $args ) {
							if ( method_exists( $this, $property ) ) {
								return call_user_func_array( [ $this, $property ], $args );
							}
							
							return $this->$property;
						}, $object, $object )->__invoke();
						
						return $value;
					};
					
					$ship_method_clone_class = new $class_name();
					
					
					$final_mapping = [];
					
					global $wpdb;
					$instance_ids = $wpdb->get_results( "SELECT instance_id FROM {$wpdb->prefix}woocommerce_shipping_zone_methods where method_id='{$method_id}'" );
					foreach ( $instance_ids as $item ) {
						$instance_id                 = $item->instance_id;
						$ship_method_options         = get_option( 'woocommerce_' . $method_id . '_' . $instance_id . '_settings' );
						$ship_method_custom_services = $ship_method_options['services'];
						
						$eu_array = $reader( $ship_method_clone_class, 'eu_array ' );
						
						if ( 'PL' === $reader( $ship_method_clone_class, 'origin_country ' ) ) {
							$use_services = $reader( $ship_method_clone_class, 'polandservices' );
						} elseif ( is_array( $eu_array ) && in_array( $reader( $ship_method_clone_class, 'origin_country' ), $eu_array ) ) {
							$use_services = $reader( $ship_method_clone_class, 'euservices' );
						} else {
							$use_services = $reader( $ship_method_clone_class, 'services' );
						}
						
						foreach ( $use_services as $id => $service ) {
							if ( ! isset( $ship_method_custom_services[ $id ] ) || empty( $ship_method_custom_services[ $id ]['enabled'] ) ) {
								unset( $use_services[ $id ] );
							}
						}
						foreach ( $use_services as $code => $title ) {
							$final_mapping[] = [
								'method_id'           => $method_id,
								'instance_id'         => $instance_id,
								'service_id'          => $code,
								'service_name'        => strtoupper( $method_id ) . ' - ' . $title,
								'service_id_on_order' => "{$method_id}:{$instance_id}:{$code}",
							];
						}
					}
					
					$default[ $method_id ]['services'] = $final_mapping;
				}
				
				return $default;
			} );
		}
		
		
		public function stop_order_from_re_syncing() {
			$order_co_options = In8_SalesOrder::getOptions( 2 );
			// add the checkbox to stop the order from syncing on the meta box
			/*add_action('in8_so_order_meta_box_after_logs', function($order_id) use ($order_co_options){
				
				$checkIfIsOn = $order_co_options['stop_orders_from_re_syncing'] && $order_co_options['stop_orders_from_re_syncing'] == 'on'
				               || !empty(get_post_meta($order_id, 'in8_do_not_sync', true));
				
				?>
					<p>
						<label for='in8_stop_order_from_syncing'>Disable syncing:</label>
						<input id='in8_stop_order_from_syncing' type='checkbox' <?=$checkIfIsOn ? 'checked' : ''?> name='in8_do_not_sync' value='1'>
					</p>
				<?php
				
			});*/
			
			add_action( 'admin_footer', function () use ( $order_co_options ) {
				if ( ! isset( $_REQUEST['post'] ) ) {
					return;
				}
				if ( ! isset( $_REQUEST['action'] ) || $_REQUEST['action'] != 'edit' ) {
					return;
				}
				
				$order_id = $_REQUEST['post'];
				if ( get_post_type( $order_id ) != 'shop_order' ) {
					return;
				}
				
				if ( ! empty( $order_co_options['stop_orders_from_re_syncing'] ) && ! empty( get_post_meta( $order_id, 'internalid', TRUE ) ) ) {
					?>
					<script>
                        jQuery(function ($) {
                            window.stopOrderFromSyncing = 1;
                            $('#manualOrderPush').click(function () {
                                if (window.stopOrderFromSyncing !== 1) return false;

                                if (confirm('The update sync is disabled, are you sure you want to update this order?')) {
                                    window.stopOrderFromSyncing = 0;
                                    $(this).trigger('click');
                                }
                            });
                        });
					</script>
					<?php
				}
			} );
			
			add_filter( 'in8_do_not_sync', function ( $default, $order_id ) use ( $order_co_options ) {
				
				// if the feature is on
				if ( ! empty( $order_co_options['stop_orders_from_re_syncing'] ) ) {
					// and the order has been already sync to ns
					if ( ! empty( get_post_meta( $order_id, 'internalid', TRUE ) ) ) {
						add_filter('in8_do_not_sync_error', function(){return 'Do not resync order. E:985';});
						// stop from updating
						return TRUE;
					}
				}
				
				
				return $default;
			}, 10, 2 );
			
		}
		
		public function map_gravity_form_pdf() {
			// check if the plugin is enabled
			if ( ! in8_is_plugin_active( 'woocommerce-gravityforms-product-addons/gravityforms-product-addons.php' ) ) {
				return;
			}
			
			add_filter( 'in8_mapping_field_value', function ( $false, $mapping, $user_id, $order ) {
				if ( ! empty( $mapping['ns_field_value'] ) && $mapping['ns_field_value'] == 'gravity_form_pdf' ) {
					$entry_id = NULL;
					$form_id  = NULL;
					
					foreach ( $order->get_items() as $item ) {
						$tempData = $item->get_meta( '_gravity_forms_history', TRUE );
						if ( ! empty( $tempData ) ) {
							$entry_id = isset( $tempData['_gravity_form_linked_entry_id'] ) ? $tempData['_gravity_form_linked_entry_id'] : NULL;
							$form_id  = isset( $tempData['_gravity_form_lead'] ) ? $tempData['_gravity_form_lead']['form_id'] : NULL;
						}
					}
					
					$form     = GFAPI::get_form( $form_id );
					$pdf_list = $form['gfpdf_form_settings'];
					if ( ! empty( $pdf_list ) ) {
						$home_url = untrailingslashit( strtok( home_url(), '?' ) );
						foreach ( $pdf_list as $pdf ) {
							$url = $home_url . '/?gpdf=1&pid=' . $pdf['id'] . '&lid=' . $entry_id . '&action=download';
							
							return 'file:' . $url . ';PDF';
						}
					}
					
					return '';
				}
				
				return $false;
				
			}, 10, 4 );
		}
		
		public function map_has_role() {
			
			add_filter( 'in8_mapping_field_value', function ( $default, $mapping, $user_id ) {
				
				if ( $mapping['type'] == 1 ) { // only for when using User Field
					if ( $mapping['wc_field_key'] == 'user_role' || $mapping['wc_field_check_key'] == 'user_role' ) {
						$user  = get_user_by( 'id', $user_id );
						$roles = apply_filters( 'in8_mapping_user_roles', $user->roles, $mapping );
						if ( empty( $roles ) ) {
							return NULL;
						}
						
						switch ( $mapping['operator'] ) {
							case 1:
								if ( in_array( $mapping['wc_field_value'], $roles ) ) {
									return $mapping['ns_field_value'];
								} else {
									return FALSE;
								}
								break;
							case 3:
								return $roles[0];
							case 4:
								if ( in_array( $mapping['wc_field_value'], $roles ) ) {
									return $mapping['wc_field_value'];
								} else {
									return FALSE;
								}
								break;
						}
					}
				}
				
				if ( $mapping['type'] == 2 ) {
					if ( preg_match( '/\.has_role/', $mapping['wc_field_key'] ) || preg_match( '/\.has_role/i', $mapping['wc_field_check_key'] ) ) {
						$meta_field = preg_match( '/\.has_role/', $mapping['wc_field_key'] ) ? str_replace( '.has_role', '', $mapping['wc_field_key'] ) : str_replace( '.has_role', '', $mapping['wc_field_check_key'] );
						$roles      = apply_filters( 'in8_mapping_user_roles', get_user_meta( $user_id, $meta_field, TRUE ), $mapping );
						if ( empty( $roles ) ) {
							return NULL;
						}
						
						switch ( $mapping['operator'] ) {
							case 1:
								if ( isset( $roles[ $mapping['wc_field_value'] ] ) && $roles[ $mapping['wc_field_value'] ] == 1 ) {
									return $mapping['wc_field_value'];
								} else {
									return FALSE;
								}
								break;
							case 3:
								reset( $roles );
								
								return key( $roles );
							case 4:
								if ( isset( $roles[ $mapping['wc_field_value'] ] ) && $roles[ $mapping['wc_field_value'] ] == 1 ) {
									return $mapping['wc_field_value'];
								} else {
									return FALSE;
								}
								break;
						}
					}
				}
				
				return $default;
			}, 10, 3 );
		}
		
		/**
		 *    This function will show a warning on the order page for the items that are missing the internalid
		 */
		public function show_internal_id_warning_on_so_page() {
			add_action( 'woocommerce_after_order_itemmeta', function ( $item_id, $item, $product ) {
				/* @var WC_Order_Item_Product $item */
				$test = false;
				$old_wc = false;
				if( is_a($item, 'WC_Order_Item_Product') ) {
					$test = $item->is_type( 'line_item' );
				}elseif(is_array($item)){
					$test = $item['type'] == 'line_item';
					$old_wc = true;
				}
				if ( $test ) {
					if( !$old_wc ) {
						$variation_id = $item->get_variation_id();
						$product_id = $item->get_product_id();
					}else{
						$variation_id = $item['variation_id'];
						$product_id = $item['product_id'];
					}
					if ( ! empty( $variation_id ) ) {
						$internalId = get_post_meta( $variation_id, 'internalid', TRUE );
					} else {
						$internalId = get_post_meta( $product_id, 'internalid', TRUE );
					}
					
					if ( empty( $internalId ) ) {
						?>
						<div class="in8-alert in8-alert-block">
							<h4 class="in8-alert-heading">Warning!</h4>
							This item does not have an internal id.
						</div>
						<?php
					}
				}
				
			}, 10000000, 3 );
		}
		
		public function map_payeezy_credit_card(){
			add_filter( 'wc_first_data_payeezy_credit_card_payment_form_default_credit_card_fields', function($default){
				
				$default['card-number']['name'] = 'wc-first-data-payeezy-credit-card-account-number';
				if ( ! empty( $default['card-csc'] ) ) {
					$default['card-csc']['name'] = 'wc-first-data-payeezy-credit-card-csc';
				}
				
				return $default;
			}, 10000 );
			
			add_filter('in8sync_so_cc_data', function($default, $order){
				if( $order->get_payment_method() != 'first_data_payeezy_credit_card' ) return $default;
				
				list( $month, $year ) = explode( "/", $_POST['wc-first-data-payeezy-credit-card-expiry'] );
				$month = trim( $month );
				$year  = trim( $year );
				
				$originalDate = $year . "-" . $month . "-01";
				$originalDate = date( "m/Y", strtotime( $originalDate ) );
				
				$default['date'] = $originalDate;
				$default['cardNumber'] = $_POST['wc-first-data-payeezy-credit-card-account-number'];
				
				return $default;
				
			}, 10, 2);
		}
		
		/**
		 * Works with this plugin: https://codecanyon.net/item/woocommerce-cybersource-payment-gateway/12199526
		 */
		public function map_cybersource_credit_card() {
			add_filter( 'in8_so_customer_after_prepared_data', function ( $data, $order ) {
				if ( $order->get_payment_method() != 'cybsawm' ) {
					return $data;
				}
				
				$plugin_settings_map     = In8_SalesOrder::getMappings();
				$co_plugins_settings_map = In8_Customer::getOptions( 2 );
				if ( ! isset( $co_plugins_settings_map['enable_ttr_cybersource_tokenization'] ) || $co_plugins_settings_map['enable_ttr_cybersource_tokenization'] != 'on' ) {
					return $data;
				}
				
				$token          = get_post_meta( $order->get_id(), '_ttr_token', TRUE );
				$card_type_name = get_post_meta( $order->get_id(), '_ttr_ns_card_type_internal_description', TRUE );
				$card_type_name = trim( str_replace( 'Tokenized', '', $card_type_name ) );
				$card_type_name = str_replace( ' ', '', $card_type_name );
				$pay_method     = strtolower( 'paymentmethod_' . $card_type_name );
				$expDate        = get_post_meta( $order->get_id(), '_ttr_card_expire_month_year', TRUE );
				//credit card array
				if ( ! empty( $token ) && ! empty( $plugin_settings_map[ $pay_method ] ) ) {
					$data['creditcards'] = [
						[
							'ccnumber'      => $token,
							'paymentmethod' => $plugin_settings_map[ $pay_method ],
							'ccexpiredate'  => $expDate,
							'ccmemo'        => 'Token',
							'ccdefault'     => "T",
						]
					];
				}
				
				return $data;
				
			}, 10, 2 );
		}
		
		public function map_new_cybersource_tokens(){
			add_filter('in8_so_token', function($default, $order){
				
				/* @var WC_Order $order */
				if( $order->get_payment_method() == 'cybersource_credit_card' ){
					$flex_token = get_post_meta( $order->get_id(), '_wc_cybersource_credit_card_flex_payment_token', TRUE );
					if( !empty($flex_token) ) return $flex_token;
					
					return get_post_meta( $order->get_id(), '_wc_cybersource_credit_card_payment_token', TRUE );
				}
				
				return $default;
			}, 10, 2);
			add_filter('in8_so_token_card_type', function($default, $order){
				
				/* @var WC_Order $order */
				if( $order->get_payment_method() == 'cybersource_credit_card' ){
					return get_post_meta( $order->get_id(), '_wc_cybersource_credit_card_card_type', TRUE );
				}
				
				return $default;
			}, 10, 2);
			add_filter('in8_so_token_expire_date', function($default, $order){
				
				/* @var WC_Order $order */
				if( $order->get_payment_method() == 'cybersource_credit_card' ){
					return get_post_meta( $order->get_id(), '_wc_cybersource_credit_card_card_expiry_date', TRUE );
				}
				
				return $default;
			}, 10, 2);
		}
		
		public function sync_customer_with_tokens(){
			add_filter('in8_do_not_sync', function($default, $order_id, $type){
				if( !In8_Customer::testFeature( 'enable_cybersource_tokenization' ) ) return $default;
				
				$order = wc_get_order($order_id);
				
				if( !in_array($order->get_payment_method(), ['cybersource', 'cybsawm']) ) return $default;
				
				$key   = 'cybersource_token';
				if( $order->get_payment_method() == 'cybsawm' ) {
					$key = '_ttr_token';
				}
				$token = get_post_meta( $order_id, $key, TRUE );
				
				if(
					in_array($order->get_payment_method(), ['cybersource', 'cybsawm'])
					&& empty($token)
				){
					return true;
				}else{// Base plugin
					if( $type == 'order' ) {
						$test = get_post_meta( $order_id, 'customerSyncToken', TRUE );
						if ( empty( $test ) ) {
							update_post_meta( $order_id, 'customerSyncToken', "T" );
							do_action( 'in8sync_async_syncOnNetsuite', $order_id, FALSE, TRUE );
						}
					}
				}
				
				return $default;
			}, 10, 3);
		}
		
	}
	
	new In8_Customers_Sales_Options();